@@ -115,12 +115,10 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     m_untranslatedGenericName = desktopGroup.readEntryUntranslated("GenericName");
     m_untranslatedName = desktopGroup.readEntryUntranslated("Name");
 
-    m_lstFormFactors = entryMap.take(QStringLiteral("X-KDE-FormFactors")).split(QLatin1Char(' '), Qt::SkipEmptyParts);
-
     if (entryMap.remove(QStringLiteral("Keywords"))) {
         m_lstKeywords = desktopGroup.readXdgListEntry("Keywords");
     }
-    m_lstKeywords += entryMap.take(QStringLiteral("X-KDE-Keywords")).split(QLatin1Char(' '), Qt::SkipEmptyParts);
+
     if (entryMap.remove(QStringLiteral("Categories"))) {
         categories = desktopGroup.readXdgListEntry("Categories");
     }
@@ -131,10 +129,6 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
 
     m_strDesktopEntryName = _name;
 
-    if (entryMap.remove(QStringLiteral("AllowDefault"))) {
-        m_bAllowAsDefault = desktopGroup.readEntry("AllowDefault", true);
-    }
-
     // Store all additional entries in the property map.
     // A QMap<QString,QString> would be easier for this but we can't
     // break BC, so we have to store it in m_mapProps.
@@ -181,32 +175,36 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
             continue;
         }
 
-        if (config->hasActionGroup(group)) {
-            const KConfigGroup cg = config->actionGroup(group);
-            if (!cg.hasKey("Name") || !cg.hasKey("Exec")) {
-                qCWarning(SERVICES) << "The action" << group << "in the desktop file" << q->entryPath() << "has no Name or no Exec key";
-            } else {
-                const QMap<QString, QString> entries = cg.entryMap();
-
-                QVariantMap entriesVariants;
-
-                for (auto it = entries.constKeyValueBegin(); it != entries.constKeyValueEnd(); ++it) {
-                    // Those are stored separately
-                    if (it->first == QLatin1String("Name") || it->first == QLatin1String("Icon") || it->first == QLatin1String("Exec")
-                        || it->first == QLatin1String("NoDisplay")) {
-                        continue;
-                    }
-
-                    entriesVariants.insert(it->first, it->second);
-                }
-
-                KServiceAction action(group, cg.readEntry("Name"), cg.readEntry("Icon"), cg.readEntry("Exec"), cg.readEntry("NoDisplay", false), serviceClone);
-                action.setData(QVariant::fromValue(entriesVariants));
-                m_actions.append(action);
-            }
-        } else {
+        if (!config->hasActionGroup(group)) {
             qCWarning(SERVICES) << "The desktop file" << q->entryPath() << "references the action" << group << "but doesn't define it";
+            continue;
         }
+
+        const KConfigGroup cg = config->actionGroup(group);
+        if (!cg.hasKey("Name")) {
+            qCWarning(SERVICES) << "The action" << group << "in the desktop file" << q->entryPath() << "has no Name key";
+            continue;
+        }
+
+        if (!cg.hasKey("Exec") && !config->desktopGroup().readEntry("DBusActivatable", false)) {
+            qCWarning(SERVICES) << "The action" << group << "in the desktop file" << q->entryPath() << "has no Exec key and not D-Bus activatable";
+            continue;
+        }
+
+        const QMap<QString, QString> entries = cg.entryMap();
+        QVariantMap entriesVariants;
+        for (const auto &[key, value] : entries.asKeyValueRange()) {
+            // Those are stored separately
+            if (key == QLatin1String("Name") || key == QLatin1String("Icon") || key == QLatin1String("Exec") || key == QLatin1String("NoDisplay")) {
+                continue;
+            }
+
+            entriesVariants.insert(key, value);
+        }
+
+        KServiceAction action(group, cg.readEntry("Name"), cg.readEntry("Icon"), cg.readEntry("Exec"), cg.readEntry("NoDisplay", false), serviceClone);
+        action.setData(QVariant::fromValue(entriesVariants));
+        m_actions.append(action);
     }
 
     serviceClone->setActions(m_actions);
@@ -214,7 +212,9 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
 
 void KServicePrivate::load(QDataStream &s)
 {
-    qint8 def;
+    qint8 unused;
+    QString unused2;
+    QStringList unused3;
     qint8 term;
     qint8 dst;
 
@@ -225,17 +225,16 @@ void KServicePrivate::load(QDataStream &s)
     // clang-format off
     s >> m_strType >> m_strName >> m_strExec >> m_strIcon
       >> term >> m_strTerminalOptions
-      >> m_strWorkingDirectory >> m_strComment >> def >> m_mapProps
-      >> m_strLibrary
+      >> m_strWorkingDirectory >> m_strComment >> unused >> m_mapProps
+      >> unused2
       >> dst
       >> m_strDesktopEntryName
       >> m_lstKeywords >> m_strGenName
       >> categories >> menuId >> m_actions
-      >> m_lstFormFactors
+      >> unused3
       >> m_untranslatedName >> m_untranslatedGenericName >> m_mimeTypes;
     // clang-format on
 
-    m_bAllowAsDefault = bool(def);
     m_bTerminal = bool(term);
 
     m_bValid = true;
@@ -244,7 +243,6 @@ void KServicePrivate::load(QDataStream &s)
 void KServicePrivate::save(QDataStream &s)
 {
     KSycocaEntryPrivate::save(s);
-    qint8 def = m_bAllowAsDefault;
     qint8 term = m_bTerminal;
     qint8 dst = 0;
 
@@ -252,9 +250,9 @@ void KServicePrivate::save(QDataStream &s)
     // !! This data structure should remain binary compatible at all times !!
     // You may add new fields at the end. Make sure to update KSYCOCA_VERSION
     // number in ksycoca.cpp
-    s << m_strType << m_strName << m_strExec << m_strIcon << term << m_strTerminalOptions << m_strWorkingDirectory << m_strComment << def << m_mapProps
-      << m_strLibrary << dst << m_strDesktopEntryName << m_lstKeywords << m_strGenName << categories << menuId << m_actions << m_lstFormFactors
-      << m_untranslatedName << m_untranslatedGenericName << m_mimeTypes;
+    s << m_strType << m_strName << m_strExec << m_strIcon << term << m_strTerminalOptions << m_strWorkingDirectory << m_strComment
+      << false /* unused */ << m_mapProps << QString() /* unused */ << dst << m_strDesktopEntryName << m_lstKeywords << m_strGenName << categories << menuId
+      << m_actions << QStringList() /* unused */ << m_untranslatedName << m_untranslatedGenericName << m_mimeTypes;
 }
 
 ////
@@ -268,7 +266,6 @@ KService::KService(const QString &_name, const QString &_exec, const QString &_i
     d->m_strExec = _exec;
     d->m_strIcon = _icon;
     d->m_bTerminal = false;
-    d->m_bAllowAsDefault = true;
 }
 
 KService::KService(const QString &_fullpath)
@@ -380,14 +377,10 @@ QVariant KServicePrivate::property(const QString &_name, QMetaType::Type t) cons
 {
     if (_name == QLatin1String("Terminal")) {
         return QVariant(m_bTerminal);
-    } else if (_name == QLatin1String("AllowAsDefault")) {
-        return QVariant(m_bAllowAsDefault);
     } else if (_name == QLatin1String("Categories")) {
         return QVariant(categories);
     } else if (_name == QLatin1String("Keywords")) {
         return QVariant(m_lstKeywords);
-    } else if (_name == QLatin1String("FormFactors")) {
-        return QVariant(m_lstFormFactors);
     }
 
     auto it = m_mapProps.constFind(_name);
@@ -755,12 +748,7 @@ QStringList KService::schemeHandlers() const
 
 QStringList KService::supportedProtocols() const
 {
-    Q_D(const KService);
-
-    QStringList ret;
-
-    ret << schemeHandlers();
-
+    QStringList ret = schemeHandlers();
     const QStringList protocols = property<QStringList>(QStringLiteral("X-KDE-Protocols"));
     for (const QString &protocol : protocols) {
         if (!ret.contains(protocol)) {
